@@ -1,11 +1,3 @@
-from .actions import *
-from .cli import *
-from .core import *
-from .formatters import *
-from .jsonnet import *
-from .jsonschema import *
-from .optionals import *
-from .util import *
 from argparse import (
     Action,
     Namespace,
@@ -17,6 +9,17 @@ from argparse import (
     ONE_OR_MORE,
     ZERO_OR_MORE,
 )
+from .actions import *
+from .cli import *
+from .core import *
+from .deprecated import *
+from .formatters import *
+from .jsonnet import *
+from .jsonschema import *
+from .optionals import *
+from .signatures import *
+from .typehints import *
+from .util import *
 
 
-__version__ = '3.8.1'
+__version__ = '3.10.0'
