@@ -1,17 +1,24 @@
 """Metods to add arguments based on class/method/function signatures."""
 
 import inspect
-from enum import Enum
-from typing import Optional, Container, Type, Callable
+import re
+from typing import Callable, Container, Optional, Type, Union
 
+from .actions import _ActionConfigLoad, _ActionHelpClassPath
+from .typehints import ActionTypeHint, is_optional
 from .util import _issubclass
-from .actions import ActionEnum, _ActionConfigLoad, _ActionHelpClassPath
-from .typing import is_optional
-from .jsonschema import ActionJsonSchema
-from .optionals import docstring_parser_support, import_docstring_parse, dataclasses_support, import_dataclasses
+from .optionals import (
+    dataclasses_support,
+    docstring_parser_support,
+    import_dataclasses,
+    import_docstring_parse,
+)
 
 
-__all__ = ['SignatureArguments']
+__all__ = [
+    'compose_dataclasses',
+    'SignatureArguments',
+]
 
 
 class SignatureArguments:
@@ -23,7 +30,9 @@ class SignatureArguments:
         nested_key: Optional[str] = None,
         as_group: bool = True,
         as_positional: bool = False,
+        required: bool = False,
         skip: Optional[Container[str]] = None,
+        fail_untyped: bool = True,
     ) -> int:
         """Adds arguments from a class based on its type hints and docstrings.
 
@@ -34,7 +43,9 @@ class SignatureArguments:
             nested_key: Key for nested namespace.
             as_group: Whether arguments should be added to a new argument group.
             as_positional: Whether to add required parameters as positional arguments.
+            required: Whether the argument group is required.
             skip: Names of parameters that should be skipped.
+            fail_untyped: Whether to raise exception if a required parameter does not have a type.
 
         Returns:
             Number of arguments added.
@@ -46,19 +57,15 @@ class SignatureArguments:
         if not inspect.isclass(theclass):
             raise ValueError('Expected "theclass" argument to be a class object.')
 
-        def docs_func(base):
-            return [base.__init__.__doc__, base.__doc__]
-
-        def sign_func(base):
-            return base.__init__
-
         return self._add_signature_arguments(inspect.getmro(theclass),
                                              nested_key,
                                              as_group,
                                              as_positional,
+                                             required,
                                              skip,
-                                             docs_func,
-                                             sign_func,
+                                             fail_untyped,
+                                             get_class_init_and_base_docstrings,
+                                             get_class_init,
                                              skip_first=True)
 
 
@@ -69,7 +76,9 @@ class SignatureArguments:
         nested_key: Optional[str] = None,
         as_group: bool = True,
         as_positional: bool = False,
+        required: bool = False,
         skip: Optional[Container[str]] = None,
+        fail_untyped: bool = True,
     ) -> int:
         """Adds arguments from a class based on its type hints and docstrings.
 
@@ -81,7 +90,9 @@ class SignatureArguments:
             nested_key: Key for nested namespace.
             as_group: Whether arguments should be added to a new argument group.
             as_positional: Whether to add required parameters as positional arguments.
+            required: Whether the argument group is required.
             skip: Names of parameters that should be skipped.
+            fail_untyped: Whether to raise exception if a required parameter does not have a type.
 
         Returns:
             Number of arguments added.
@@ -102,7 +113,9 @@ class SignatureArguments:
                                              nested_key,
                                              as_group,
                                              as_positional,
+                                             required,
                                              skip,
+                                             fail_untyped,
                                              skip_first=skip_first)
 
 
@@ -112,7 +125,9 @@ class SignatureArguments:
         nested_key: Optional[str] = None,
         as_group: bool = True,
         as_positional: bool = False,
+        required: bool = False,
         skip: Optional[Container[str]] = None,
+        fail_untyped: bool = True,
     ) -> int:
         """Adds arguments from a function based on its type hints and docstrings.
 
@@ -123,7 +138,9 @@ class SignatureArguments:
             nested_key: Key for nested namespace.
             as_group: Whether arguments should be added to a new argument group.
             as_positional: Whether to add required parameters as positional arguments.
+            required: Whether the argument group is required.
             skip: Names of parameters that should be skipped.
+            fail_untyped: Whether to raise exception if a required parameter does not have a type.
 
         Returns:
             Number of arguments added.
@@ -139,7 +156,9 @@ class SignatureArguments:
                                              nested_key,
                                              as_group,
                                              as_positional,
-                                             skip)
+                                             required,
+                                             skip,
+                                             fail_untyped)
 
 
     def _add_signature_arguments(
@@ -148,7 +167,9 @@ class SignatureArguments:
         nested_key: Optional[str],
         as_group: bool,
         as_positional: bool,
+        required: bool,
         skip: Optional[Container[str]],
+        fail_untyped: bool,
         docs_func: Callable = lambda x: [x.__doc__],
         sign_func: Callable = lambda x: x,
         skip_first: bool = False,
@@ -160,7 +181,9 @@ class SignatureArguments:
             nested_key: Key for nested namespace.
             as_group: Whether arguments should be added to a new argument group.
             as_positional: Whether to add required parameters as positional arguments.
+            required: Whether the argument group is required.
             skip: Names of parameters that should be skipped.
+            fail_untyped: Whether to raise exception if a required parameter does not have a type.
             docs_func: Function that returns docstrings for a given object.
             sign_func: Function that returns signature method for a given object.
             skip_first: Whether to skip first argument, i.e., skip self of class methods.
@@ -193,14 +216,12 @@ class SignatureArguments:
         doc_group, doc_params = self._gather_docstrings(objects, docs_func)
 
         ## Create group if requested ##
-        group = self._create_group_if_requested(objects[0], nested_key, as_group, doc_group)
+        group = self._create_group_if_requested(objects[0], nested_key, as_group, required, doc_group)
 
         ## Add objects arguments ##
         added_args = set()
         if skip is None:
             skip = set()
-        if dataclasses_support:
-            dataclasses = import_dataclasses('_add_signature_arguments')
         for obj, (add_args, add_kwargs) in zip(objects, add_types):
             for num, param in enumerate(inspect.signature(sign_func(obj)).parameters.values()):
                 name = param.name
@@ -211,6 +232,7 @@ class SignatureArguments:
                 skip_message = 'Skipping parameter "'+name+'" from "'+obj.__name__+'" because of: '
                 if kind in {kinds.VAR_POSITIONAL, kinds.VAR_KEYWORD} or \
                    (is_required and skip_first and num == 0) or \
+                   (not is_required and name[0] == '_') or \
                    (annotation == inspect._empty and not is_required and default is None):  # type: ignore
                     continue
                 elif is_required and not add_args:
@@ -222,7 +244,7 @@ class SignatureArguments:
                 elif name in skip:
                     self.logger.debug(skip_message+'Parameter requested to be skipped.')  # type: ignore
                     continue
-                if dataclasses_support and default.__class__ == dataclasses._HAS_DEFAULT_FACTORY_CLASS:
+                if is_factory_class(default):
                     default = obj.__dataclass_fields__[name].default_factory()
                 if annotation == inspect._empty and not is_required:  # type: ignore
                     annotation = type(default)
@@ -233,14 +255,15 @@ class SignatureArguments:
                         annotation = Optional[annotation]
                 elif not as_positional:
                     kwargs['required'] = True
+                is_subclass_typehint = False
                 if annotation in {str, int, float, bool} or \
-                   _issubclass(annotation, (str, int, float)):
+                   _issubclass(annotation, (str, int, float)) or \
+                   is_pure_dataclass(annotation):
                     kwargs['type'] = annotation
-                elif _issubclass(annotation, Enum):
-                    kwargs['action'] = ActionEnum(enum=annotation)
-                else:
+                elif annotation != inspect._empty:  # type: ignore
                     try:
-                        kwargs['action'] = ActionJsonSchema(annotation=annotation, enable_path=False)
+                        kwargs['action'] = ActionTypeHint(typehint=annotation)
+                        is_subclass_typehint = ActionTypeHint.is_subclass_typehint(annotation)
                     except ValueError as ex:
                         self.logger.debug(skip_message+str(ex))  # type: ignore
                 if 'type' in kwargs or 'action' in kwargs:
@@ -249,19 +272,80 @@ class SignatureArguments:
                         self.logger.debug(skip_message+'Argument already added.')  # type: ignore
                     else:
                         opt_str = dest if is_required and as_positional else '--'+dest
+                        if is_subclass_typehint:
+                            group.add_argument('--'+dest+'.help', action=_ActionHelpClassPath(baseclass=annotation))
                         group.add_argument(opt_str, **kwargs)
                         added_args.add(dest)
-                elif is_required:
+                elif is_required and fail_untyped:
                     raise ValueError('Required parameter without a type for '+obj.__name__+' parameter '+name+'.')
 
         return len(added_args)
 
 
+    def add_dataclass_arguments(
+        self,
+        theclass: Type,
+        nested_key: str,
+        default: Union[Type, dict] = None,
+        as_group: bool = True,
+        required: bool = False,
+        **kwargs
+    ):
+        """Adds arguments from a dataclass based on its field types and docstrings.
+
+        Args:
+            theclass: Class from which to add arguments.
+            nested_key: Key for nested namespace.
+            default: Vale for defaults. Must be instance of or kwargs for theclass.
+            as_group: Whether arguments should be added to a new argument group.
+            required: Whether the argument group is required.
+
+        Returns:
+            Number of arguments added.
+
+        Raises:
+            ValueError: When not given a dataclass.
+            ValueError: When default is not instance of or kwargs for theclass.
+        """
+        dataclasses = import_dataclasses('add_dataclass_arguments')
+        if not is_pure_dataclass(theclass):
+            raise ValueError('Expected "theclass" argument to be a pure dataclass, given '+str(theclass))
+
+        doc_group, doc_params = self._gather_docstrings([theclass], get_class_init_and_base_docstrings)
+        for key in ['help', 'title']:
+            if key in kwargs and kwargs[key] is not None:
+                doc_group = strip_title(kwargs.pop(key))
+        group = self._create_group_if_requested(theclass, nested_key, as_group, required, doc_group, config_load=True, config_load_type=theclass)
+
+        defaults = {}
+        if default is not None:
+            if isinstance(default, dict):
+                try:
+                    default = theclass(**default)
+                except TypeError:
+                    pass
+            if not isinstance(default, theclass):
+                raise ValueError('Expected "default" argument to be an instance of "'+theclass.__name__+'" or its kwargs dict, given '+str(default))
+            defaults = dataclasses.asdict(default)
+
+        for field in dataclasses.fields(theclass):
+            arg_kwargs = dict(kwargs)
+            arg_kwargs.update({
+                'type': field.type,
+                'default': defaults.get(field.name),
+                'help': doc_params.get(field.name),
+            })
+            group.add_argument('--' + nested_key + '.' + field.name, **arg_kwargs)
+
+        return len(dataclasses.fields(theclass))
+
+
     def add_subclass_arguments(
         self,
         baseclass: Type,
         nested_key: str,
         as_group: bool = True,
+        required: bool = False,
         metavar: str = '{"class_path":...[,"init_args":...]}',
         help: str = 'Dictionary with "class_path" and "init_args" for any subclass of %(baseclass_name)s.',
         **kwargs
@@ -279,6 +363,7 @@ class SignatureArguments:
             baseclass: Base class to use to check subclasses.
             nested_key: Key for nested namespace.
             as_group: Whether arguments should be added to a new argument group.
+            required: Whether the argument group is required.
             metavar: Variable string to show in the argument's help.
             help: Description of argument to show in the help.
 
@@ -288,11 +373,8 @@ class SignatureArguments:
         if not inspect.isclass(baseclass):
             raise ValueError('Expected "baseclass" argument to be a class object.')
 
-        def docs_func(base):
-            return [base.__init__.__doc__, base.__doc__]
-
-        doc_group = self._gather_docstrings([baseclass], docs_func)[0]
-        group = self._create_group_if_requested(baseclass, nested_key, as_group, doc_group, config_load=False)
+        doc_group = self._gather_docstrings([baseclass], get_class_init_and_base_docstrings)[0]
+        group = self._create_group_if_requested(baseclass, nested_key, as_group, required, doc_group, config_load=False)
 
         group.add_argument('--'+nested_key+'.help', action=_ActionHelpClassPath(baseclass=baseclass))
         group.add_argument(
@@ -322,10 +404,15 @@ class SignatureArguments:
                         for param in docstring.params:
                             if param.arg_name not in doc_params:
                                 doc_params[param.arg_name] = param.description
-        return doc_group, doc_params
+        return strip_title(doc_group), doc_params
 
 
-    def _create_group_if_requested(self, obj, nested_key, as_group, doc_group, config_load=True):
+    def _create_group_if_requested(self, obj, nested_key, as_group, required, doc_group, config_load=True, config_load_type=None):
+        if required:
+            if nested_key is None:
+                raise ValueError('A nested_key is mandatory to make required.')
+            self.required_args.add(nested_key)
+
         group = self
         if as_group:
             if doc_group is None:
@@ -333,5 +420,48 @@ class SignatureArguments:
             name = obj.__name__ if nested_key is None else nested_key
             group = self.add_argument_group(doc_group, name=name)
             if config_load and nested_key is not None:
-                group.add_argument('--'+nested_key, action=_ActionConfigLoad)
+                group.add_argument('--'+nested_key, action=_ActionConfigLoad(basetype=config_load_type))
         return group
+
+
+def get_class_init_and_base_docstrings(value):
+    return [value.__init__.__doc__, value.__doc__]
+
+
+def get_class_init(value):
+    return value.__init__
+
+
+def strip_title(value):
+    if value is not None:
+        return re.sub(r'\.$', '', value.strip())
+
+
+def is_factory_class(value):
+    result = False
+    if dataclasses_support:
+        dataclasses = import_dataclasses('is_default_factory_class')
+        result = value.__class__ == dataclasses._HAS_DEFAULT_FACTORY_CLASS
+    return result
+
+
+def is_pure_dataclass(value):
+    if not dataclasses_support or not inspect.isclass(value):
+        return False
+    dataclasses = import_dataclasses('is_pure_dataclass')
+    classes = [c for c in inspect.getmro(value) if c != object]
+    return all(dataclasses.is_dataclass(c) for c in classes)
+
+
+def compose_dataclasses(*args):
+    """Returns a pure dataclass inheriting all given dataclasses and properly handling __post_init__."""
+    dataclasses = import_dataclasses('compose_dataclasses')
+
+    @dataclasses.dataclass
+    class ComposedDataclass(*args):
+        def __post_init__(self):
+            for arg in args:
+                if hasattr(arg, '__post_init__'):
+                    arg.__post_init__(self)
+
+    return ComposedDataclass
