@@ -71,6 +71,24 @@ def _load_config(value, enable_path=True, flat_namespace=True):
     return value, cfg_path
 
 
+def get_key_value_from_flat_dict(cfg, key):
+    value = cfg.get(key)
+    if value is not None:
+        return value
+    value = {k[len(key)+1:]: v for k, v in cfg.items() if k.startswith(key+'.')}
+    return _flat_namespace_to_dict(Namespace(**value))
+
+
+def update_key_value_in_flat_dict(cfg, key, value):
+    if isinstance(value, dict):
+        value = vars(_dict_to_flat_namespace(value))
+        if key in cfg:
+            del cfg[key]
+        cfg.update({key+'.'+k: v for k, v in value.items()})
+    else:
+        cfg[key] = value
+
+
 def _get_key_value(cfg, key, parent=False):
     """Gets the value for a given key in a config object (dict or argparse.Namespace)."""
     def key_in_cfg(cfg, key):
@@ -81,7 +99,7 @@ def _get_key_value(cfg, key, parent=False):
 
     c = cfg
     k = key
-    while '.' in key and not key_in_cfg(c, k):
+    while '.' in k and not key_in_cfg(c, k):
         kp, k = k.split('.', 1)
         c = c[kp] if isinstance(c, dict) else getattr(c, kp)
 
@@ -101,9 +119,14 @@ def _flat_namespace_to_dict(cfg_ns:Namespace) -> Dict[str, Any]:
     def raise_conflicting_base(base):
         raise ParserError('Conflicting namespace base: '+base)
 
+    nested_keys = {k.rsplit('.', 1)[0]+'.' for k in vars(cfg_ns).keys() if '.' in k}
+    skip_keys = {k for k, v in vars(cfg_ns).items() if v is None and k+'.' in nested_keys}
+
     cfg_ns = deepcopy(cfg_ns)
     cfg_dict = {}
     for k, v in vars(cfg_ns).items():
+        if k in skip_keys:
+            continue
         ksplit = k.split('.')
         if len(ksplit) == 1:
             if k in cfg_dict:
@@ -203,14 +226,14 @@ def namespace_to_dict(cfg_ns:Namespace) -> Dict[str, Any]:
     return expand_namespace(cfg_ns)
 
 
-def strip_meta(cfg:Union[Namespace, Dict]):
+def strip_meta(cfg:Union[Namespace, Dict[str, Any]]) -> Dict[str, Any]:
     """Removes all metadata keys from a configuration object.
 
     Args:
         cfg: The configuration object to strip.
 
     Returns:
-        argparse.Namespace: The stripped configuration object.
+        A copy of the configuration object without any metadata keys.
     """
     cfg = deepcopy(cfg)
     if not isinstance(cfg, dict):
@@ -260,6 +283,8 @@ def _issubclass(cls, class_or_tuple):
 
 def import_object(name):
     """Returns an object in a module given its dot import path."""
+    if not isinstance(name, str):
+        raise ValueError('Expected a dot import path string')
     name_module, name_object = name.rsplit('.', 1)
     module = __import__(name_module, fromlist=[name_object])
     return getattr(module, name_object)
@@ -276,13 +301,14 @@ def _suppress_stderr():
 @contextmanager
 def change_to_path_dir(path):
     """A context manager for running code in the directory of a path."""
-    if not path.is_url:
+    chdir = path is not None and not path.is_url
+    if chdir:
         cwd = os.getcwd()
-        os.chdir(os.path.abspath(os.path.join(str(path), os.pardir)))
+        os.chdir(os.path.abspath(os.path.dirname(str(path))))
     try:
         yield None
     finally:
-        if not path.is_url:
+        if chdir:
             os.chdir(cwd)
 
 
@@ -320,9 +346,6 @@ class Path:
         if cwd is None:
             cwd = os.getcwd()
 
-        if isinstance(cwd, list):
-            cwd = cwd[0]  # Temporal until multiple cwds is implemented.
-
         is_url = False
         if isinstance(path, Path):
             is_url = path.is_url
@@ -394,7 +417,7 @@ class Path:
 
     def __repr__(self):
         cwd = '' if self.rel_path == self.abs_path else ', cwd='+self.cwd
-        return 'Path('+self.rel_path+cwd+')'
+        return 'Path_'+self.mode+'('+self.rel_path+cwd+')'
 
     def __call__(self, absolute:bool=True) -> str:
         """Returns the path as a string.
