@@ -10,6 +10,9 @@ class CLITests(unittest.TestCase):
             return a1
 
         self.assertEqual(1.2, CLI(function, args=['1.2']))
+        parser = CLI(function, return_parser=True, set_defaults={'a1': 3.4})
+        self.assertIsInstance(parser, ArgumentParser)
+        self.assertEqual(3.4, parser.get_defaults().a1)
 
 
     def test_multiple_functions_cli(self):
@@ -22,6 +25,9 @@ class CLITests(unittest.TestCase):
         functions = [cmd1, cmd2]
         self.assertEqual(5, CLI(functions, args=['cmd1', '5']))
         self.assertEqual('Y', CLI(functions, args=['cmd2', '--a2=Y']))
+        parser = CLI(functions, return_parser=True, set_defaults={'cmd2.a2': 'Z'})
+        self.assertIsInstance(parser, ArgumentParser)
+        self.assertEqual('Z', parser.parse_args(['cmd2'])['cmd2']['a2'])
 
 
     def test_single_class_cli(self):
