@@ -1,18 +1,17 @@
 #!/usr/bin/env python3
 
+import calendar
 import json
 import yaml
-import calendar
 from enum import Enum
 from io import StringIO
 from contextlib import redirect_stdout
-from typing import Dict, List, Tuple, Optional, Union, Any
+from typing import Any, Dict, List, Optional, Tuple, Union
 from jsonargparse_tests.base import *
 from jsonargparse.actions import _find_action
 from jsonargparse.util import _suppress_stderr
 
 
-@unittest.skipIf(not jsonschema_support, 'jsonschema package is required')
 class SignaturesTests(unittest.TestCase):
 
     def test_add_class_arguments(self):
@@ -109,7 +108,6 @@ class SignaturesTests(unittest.TestCase):
         self.assertEqual('a', Class3(**namespace_to_dict(cfg))())
 
         self.assertRaises(ParserError, lambda: parser.parse_args([]))  # c3_a0 is required
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--c3_a0=0', '--c3_a4=4.0']))  # c3_a4 is str or None
         self.assertRaises(ParserError, lambda: parser.parse_args(['--c3_a0=0', '--c3_a7=["3", "3", 3.0]']))  # tuple[1] is int
 
         if docstring_parser_support:
@@ -263,8 +261,22 @@ class SignaturesTests(unittest.TestCase):
         self.assertIn('--cal.init_args.firstweekday', out.getvalue())
 
 
+    def test_required_group(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_subclass_arguments(calendar.Calendar, 'cal', required=True)
+        self.assertRaises(ParserError, lambda: parser.parse_args([]))
+        self.assertRaises(ValueError, lambda: parser.add_class_arguments(calendar.Calendar, required=True))
+
+
+    def test_invalid_type(self):
+
+        def func(a1: None):
+            return a1
+
+        parser = ArgumentParser(error_handler=None)
+        self.assertRaises(ValueError, lambda: parser.add_function_arguments(func))
+
 
-    @unittest.skipIf(not jsonschema_support, 'jsonschema package is required')
     def test_optional_enum(self):
 
         class MyEnum(Enum):
@@ -402,11 +414,11 @@ class SignaturesTests(unittest.TestCase):
         init_args = '"init_args": {"a1": 7}'
         cfg = parser.parse_args(['--c1={'+class_path+', '+init_args+'}'])
         self.assertEqual(cfg.c1.class_path, 'jsonargparse_tests.signatures_tests.Class1')
-        self.assertEqual(cfg.c1.init_args, Namespace(a1=7))
+        self.assertEqual(cfg.c1.init_args, Namespace(a1=7, a2=2.3))
         cfg = parser.instantiate_subclasses(cfg)
-        self.assertIsInstance(cfg.c1, Class1)
-        self.assertEqual(7, cfg.c1.a1)
-        self.assertEqual(2.3, cfg.c1.a2)
+        self.assertIsInstance(cfg['c1'], Class1)
+        self.assertEqual(7, cfg['c1'].a1)
+        self.assertEqual(2.3, cfg['c1'].a2)
 
 
     def test_implicit_optional(self):
@@ -442,7 +454,106 @@ class SignaturesTests(unittest.TestCase):
             self.assertNotIn('a1 description', help_str.getvalue())
 
 
-@unittest.skipIf(not jsonschema_support, 'jsonschema package is required')
+    def test_print_config(self):
+        class MyClass:
+            def __init__(
+                self,
+                a1: calendar.Calendar,
+                a2: int = 7,
+            ):
+                pass
+
+        parser = ArgumentParser()
+        parser.add_class_arguments(MyClass, 'g')
+
+        out = StringIO()
+        with redirect_stdout(out), self.assertRaises(SystemExit):
+            parser.parse_args(['--g.a1=calendar.Calendar', '--print_config'])
+
+        outval = yaml.safe_load(out.getvalue())
+        self.assertEqual(outval['g'], {'a1': {'class_path': 'calendar.Calendar', 'init_args': {'firstweekday': 0}}, 'a2': 7})
+
+
+    def test_link_arguments(self):
+
+        class ClassA:
+            def __init__(self, v1: int = 2, v2: int = 3):
+                pass
+
+        class ClassB:
+            def __init__(self, v1: int = -1, v2: int = 4, v3: int = 2):
+                """ClassB title
+                Args:
+                    v1: b v1 help
+                """
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_class_arguments(ClassA, 'a')
+        parser.add_class_arguments(ClassB, 'b')
+        parser.link_arguments('a.v2', 'b.v1')
+        def add(*args):
+            return sum(args)
+        parser.link_arguments(('a.v1', 'a.v2'), 'b.v2', add)
+
+        cfg = parser.parse_args([])
+        self.assertEqual(cfg.b.v1, cfg.a.v2)
+        self.assertEqual(cfg.b.v2, cfg.a.v1+cfg.a.v2)
+        cfg = parser.parse_args(['--a.v1=11', '--a.v2=7'])
+        self.assertEqual(7, cfg.b.v1)
+        self.assertEqual(11+7, cfg.b.v2)
+        self.assertEqual(Namespace(), parser.parse_args([], defaults=False))
+
+        self.assertRaises(ParserError, lambda: parser.parse_args(['--b.v1=5']))
+        self.assertRaises(ValueError, lambda: parser.link_arguments('a.v2', 'b.v1'))
+        self.assertRaises(ValueError, lambda: parser.link_arguments('x', 'b.v2'))
+        self.assertRaises(ValueError, lambda: parser.link_arguments('a.v1', 'x'))
+        self.assertRaises(ValueError, lambda: parser.link_arguments(('a.v1', 'a.v2'), 'b.v3'))
+
+        help_str = StringIO()
+        parser.print_help(help_str)
+        self.assertIn('Linked arguments', help_str.getvalue())
+        self.assertIn('b.v1 <= a.v2', help_str.getvalue())
+        self.assertIn('b.v2 <= add(a.v1, a.v2)', help_str.getvalue())
+        if docstring_parser_support:
+            self.assertIn('b v1 help', help_str.getvalue())
+
+
+    def test_link_arguments_subclasses(self):
+        class ClassA:
+            def __init__(
+                self,
+                v1: Union[int, str] = 1,
+                v2: Union[int, str] = 2,
+            ):
+                pass
+
+        from jsonargparse_tests import signatures_tests
+        setattr(signatures_tests, 'ClassA', ClassA)
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_subclass_arguments(ClassA, 'a')
+        parser.add_subclass_arguments(calendar.Calendar, 'c')
+
+        def add(v1, v2):
+            return v1 + v2
+        parser.link_arguments(('a.init_args.v1', 'a.init_args.v2'), 'c.init_args.firstweekday', add)
+
+        a_value = {
+            'class_path': 'jsonargparse_tests.signatures_tests.ClassA',
+            'init_args': {'v2': 3},
+        }
+
+        cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
+        self.assertEqual(cfg.c.init_args.firstweekday, 4)
+        self.assertEqual(cfg.c.init_args.firstweekday, cfg.a.init_args.v1+cfg.a.init_args.v2)
+
+        self.assertRaises(ValueError, lambda: parser.link_arguments('a.init_args.v1', 'c'))
+
+        a_value['init_args'] = {'v1': 'a', 'v2': 'b'}
+        with self.assertRaises(ParserError):
+            parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
+
+
 class SignaturesConfigTests(TempDirTestCase):
 
     def test_add_function_arguments_config(self):
